#ifndef CUSTOMEDIT_H_
#define CUSTOMEDIT_H_

#include "mythplayer.h"
#include "programinfo.h"
#include "mythscreentype.h"

class MythUITextEdit;
class MythUIButton;
class MythUIButtonList;
class MythUIButtonListItem;

/**
 * A screen to create a fully custom recording.  The recording will be
 * created as a power search.
 */
class CustomEdit : public MythScreenType
{
    Q_OBJECT
  public:

    explicit CustomEdit(MythScreenStack *parent, ProgramInfo *m_pginfo = nullptr);
   ~CustomEdit(void);

    bool Create() override; // MythScreenType
    bool keyPressEvent(QKeyEvent *) override; // MythScreenType
    void customEvent(QEvent *event) override; // MythUIType

  protected slots:
    void ruleChanged(MythUIButtonListItem *item);
    void textChanged(void);
    void clauseChanged(MythUIButtonListItem *item);
    void clauseClicked(MythUIButtonListItem *item);
    void testClicked(void);
    void recordClicked(void);
    void storeClicked(void);
    void scheduleCreated(int);

  private:
    void loadData(void);
    void loadClauses(void);
    bool checkSyntax(void);
    void storeRule(bool is_search, bool is_new);
    void deleteRule(void);
    QString evaluate(QString clause);

    ProgramInfo *m_pginfo;
    QString m_baseTitle;

    int m_maxex;
    bool m_evaluate;

    QString m_seSuffix;
    QString m_exSuffix;

    MythUIButtonList *m_ruleList;
    MythUIButtonList *m_clauseList;

    MythUITextEdit *m_titleEdit;

    // Contains the SQL statement
    MythUITextEdit *m_descriptionEdit;

    // Contains the additional SQL tables
    MythUITextEdit *m_subtitleEdit;

    MythUIText   *m_clauseText;
    MythUIButton *m_testButton;
    MythUIButton *m_recordButton;
    MythUIButton *m_storeButton;
    MythUIButton *m_cancelButton;

    const MythUIButtonListItem *m_currentRuleItem;
};

struct CustomRuleInfo {
    QString recordid;
    QString title;
    QString subtitle;
    QString description;
};

Q_DECLARE_METATYPE(CustomRuleInfo)

#endif
